#!/usr/bin/env python
# -*- coding: utf-8 -*-
from tools import os_tool
import os

############################
# 初始化工程目录
############################
root_path = os_tool.get_root_path()

content = """[te_url]
mall = http://api.xuepl.com.cn:28019
admin = http://api.xuepl.com.cn:27019
srb = http://srb.xuepl.com.cn:81
[re_url]
mall = http://1.15.183.200:28019
admin = http://1.15.183.200:27019


[te_mall_db]
host = api.xuepl.com.cn
user = root
password = SongLin2021
db = mall
port = 3306

[te_admin_db]
host = api.xuepl.com.cn
user = root
password = SongLin2021
db = mall
port = 3306

[re_mall_db]
host = 1.15.183.200
user = root
password = SongLinxy
db = mall
port = 3306

[re_admin_db]
host = 1.15.183.200
user = root
password = SongLinxy
db = mall
port = 3306

[root_path]
path = E:\\case



[user_info]
login_name = 13090237226
password = e10adc3949ba59abbe56e057f20f883e"""
os_tool.mkfile(root_path, *['./', 'config.ini'], content=content)

content = """# !/usr/bin/env python
# -*-coding:utf-8 -*-

'''
# File       : conftest.py.py
# Time       ：2022-03-27 17:01
# Author     ：xuepl
# version    ：python 3.7.1
'''

# 1、注册自定义命令行参数
import os

import pytest

from tools import log
from tools.utils import get_cases


def pytest_addoption(parser):
    parser.addoption("--env", action="store",  # append
                     default="te",
                     help="指定测试环境 te, re ,pe，默认为：te")


# 2、获取命令行参数的值
@pytest.fixture(scope="session", autouse=True)
def env(request):
    # 获取环境号
    envNo = request.config.getoption("--env")
    log.info(f"------------运行环境：{envNo}----------")
    if envNo not in ["te", "re", "pe"]:  # 判断获取到的环境号，是否存在于环境列表中
        pytest.exit("请输入正确的环境号！")  # 直接结束，不运行测试用例
    # 把环境号写入环境变量
    os.environ.setdefault("AUTO_ENV", envNo)


def pytest_collection_modifyitems(session, config, items) -> None:
    # item表示每个测试用例，解决用例名称中文显示问题
    for item in items:
        item.name = item.name.encode("utf-8").decode("unicode-escape")
        item._nodeid = item._nodeid.encode("utf-8").decode("unicode-escape")


@pytest.hookimpl(hookwrapper=True, tryfirst=True)
def pytest_runtest_makereport(item, call):
    # 获取钩子方法的调用结果
    out = yield
    # print('用例执行结果', out)
    # 从钩子方法的调用结果中获取测试报告
    report = out.get_result()
    # when 常见有三 setup call teardown
    # outcome 常见值有三  passed 通过  failed 执行失败的用例 skipped 跳过执行的用例（包括所有用装饰器标记的）
    # duration 用例的执行耗时
    # 执行时间，用例路径，用例执行结果。执行通过打印用例耗时。如何执行不通过，打印报错信息
    if report.when == "call":
        if report.outcome == 'failed':
            log.error(f"用例执行路径：{report.location[0]}::{report.location[2]},用例执行结果：{report.outcome}\\n{report.longrepr}")
        else:
            log.info(
                f"用例执行路径：{report.location[0]}::{report.location[2]},用例执行结果：{report.outcome},用例执行耗时为：{report.duration}")


# 使用钩子函数，实现自动参数化
def pytest_generate_tests(metafunc):
    ''' generate (multiple) parametrized calls to a test function.'''
    ids, cases = get_cases()
    if "case" in metafunc.fixturenames:
        metafunc.parametrize("case",
                             cases,
                             ids=ids,
                             scope="function")
"""
os_tool.mkfile(root_path, *['./', 'conftest.py'], content=content)

content = """# !/usr/bin/env python
# -*-coding:utf-8 -*-

'''
# File       : run.py
# Time       ：2022-05-08 11:28
# Author     ：xuepl
# version    ：python 3.7.1
'''
import os

import pytest

report_data = "report/json"
pytest.main(["--alluredir", report_data, '--env', 'te'])
"""
os_tool.mkfile(root_path, *['./', 'run.py'], content=content)

content = """# !/usr/bin/env python
# -*-coding:utf-8 -*-

'''
# File       : test_run.py
# Time       ：2022-05-08 11:27
# Author     ：xuepl
# version    ：python 3.7.1
'''
from common.send_request import SendRequest


def test_run(case):
    SendRequest(case).send()
"""
os_tool.mkfile(root_path, *['./', 'test_run.py'], content=content)



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : send_request.py
# Time       ：2022-04-10 14:13
# Author     ：xuepl
# version    ：python 3.7.1
"""
import json
import os

import allure
import requests

from common.base_request_des import BaseRequest
from common.base_response import BaseResponse
from common.key_driver import KeyDriver


class SendRequest():
    __session = None
    local_vars = {}

    def __init__(self, case):
        self.request = BaseRequest()  # 实例化BaseRequest对象
        self.request_data = {}
        self.request.url = (case["url"], case["app"])
        self.allure_data = {"feature": case["app"]}
        for key in case:  # 遍历dict 对BaseRequest对象 赋值
            if key in ["pre_process"]:
                self.pre_process = case["pre_process"]
            elif key in ["assert"]:
                self.assert_data = case["assert"]
            elif key in ["post_process"]:
                self.post_process = case["post_process"]
            elif key in ['app', 'method', 'params', 'data', 'json', 'headers', 'files']:
                self.request_data[key] = case[key]
            elif key in ["story", "title"]:
                self.allure_data[key] = case[key]
        # 添加allure报告数据
        for key in self.allure_data:
            getattr(allure.dynamic, key)(self.allure_data[key])

    # 扩展
    # 1、session发送
    # 2、请求头中的默认值

    @property
    def session(self):
        if self.__session is None:
            __class__.__session = requests.Session()
            self.__session.headers.update({
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.127 Safari/537.36"
            })
        return self.__session

    def send(self):
        # 第一步，执行前置操作
        with allure.step("第一步、执行前置操作"):
            KeyDriver(self, self.pre_process)  # 关键字驱动
        # 第二步，发送请求
        with allure.step("第二步、发送请求"):
            allure.attach(json.dumps(self.request_data, ensure_ascii=False, indent=2), "请求数据",
                          attachment_type=allure.attachment_type.TEXT)
            self.request_data = json.loads(str(KeyDriver(self, self.request_data)))  # 关键字驱动
            for key in self.request_data:
                setattr(self.request, key, self.request_data[key])  # 使用反射的方法，对request对象进行赋值
            # 发送请求
            r = self.session.request(
                method=self.request.method,
                url=self.request.url,
                params=self.request.params,
                data=self.request.data,
                json=self.request.json,
                headers=self.request.headers,
                files=self.request.files
            )
            # 初始化为响应数据类
            self.response = BaseResponse(r)
            # 添加请求和响应数据，至测试报告中
            allure.attach(str(self.response), "响应数据",
                          attachment_type=allure.attachment_type.TEXT)
        # 第三步，执行后置操作
        with allure.step("第三步，执行后置操作"):
            KeyDriver(self, self.post_process)  # 关键字驱动
        # 第四步、断言
        with allure.step("第四步、断言"):
            KeyDriver(self, self.assert_data)  # 关键字驱动



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : log.py
# Time       ：2021/4/7 17:42
# Author     ：xuepl
# version    ：python 3.7
"""

import logging
import os
from logging.handlers import TimedRotatingFileHandler

'''
封装log方法
'''
logger = logging.getLogger(__name__)  # 获取日志记录器
logger.setLevel(level=logging.DEBUG)  # 设置日志记录器收集日志的最低级别DEBUG

root_path = os.path.join(os.getcwd(), "logs/")  # 确定日志输出的文件夹
if not os.path.exists(root_path):  # 如果log文件夹不存在，就去创建一个
    os.makedirs(root_path)
# 创建文件handler，
handler = TimedRotatingFileHandler(root_path + 'debug.log', when='d', interval=1, backupCount=30, encoding='utf-8')
handler.setLevel(logging.DEBUG)  # 设置该handler输出日志的最低级别为DEBUG
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')  # 日志格式化器，规定日志输出的格式
handler.setFormatter(formatter)
logger.addHandler(handler)

handler2 = TimedRotatingFileHandler(root_path + 'error.log', when='d', interval=1, backupCount=30, encoding='utf-8')
handler2.setLevel(logging.ERROR)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler2.setFormatter(formatter)
logger.addHandler(handler2)

# handler 控制日志输出到控制台
handler3 = logging.StreamHandler()
handler3.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler3.setFormatter(formatter)
logger.addHandler(handler3)


def info(msg):
    logger.info(msg)


def debug(msg):
    logger.debug(msg)


def warning(msg):
    logger.warning(msg)


def error(msg):
    logger.error(msg)


def critical(msg):
    logger.critical(msg)



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : response_tool.py
# Time       ：2022-04-23 9:39
# Author     ：xuepl
# version    ：python 3.7.1
"""
import json
import random
import re

import allure

from tools import log

'''
响应数据的提取

根据关键字，从响应中提取内容
提取方法：json表达式，正则表达式
响应数据的传递问题

下个接口中，怎么去获取提取出的数据
类变量做一个全局的数据变量
在定义一个从类变量中获取数据的关键字

$      代表json数据根节点
.或[]  获取对象中的节点  $['data'] 或 $.data
?()    跟判断条件  $.data.list[?(@.goodsName=='蔡徐坤')].goodsId
@      代表当前节点
[]     数据的迭代标识   $.data.list[0].goodsId
...    表示获取json中所有的节点  $...goodsId
[,]    支持迭代器中多选    $.data.list[0,1]['goodsId']
()     支持数据计算       $.data.list[(@.length-2)]['goodsId']
*      通配符代表任意字符       $.data.list[*].goodsId

python中，对jsonpath进行解析的模块叫做jsonpath
pip install jsonpath

使用
jsonpath.jsonpath(python字典或列表,json表达式)
返回值 jsonpath表达式不正确返回，False，匹配到数据返回列表
'''
import jsonpath


#
# d = {
#     "name": "王大锤",
#     "age": "12",
#     "sex": "男",
#     "addr": "上海闵行",
# }
# print(jsonpath.jsonpath(d, '$["name"]'))


def json_excutor(send_request, var_name, index, *json_path):
    """

    :param send_request:
    :param var_name: 变量名
    :param index: <=0 表示随机，如果为1 表示获取第1个数据，为2获取第二个数据依次类推
    :param *json_path: jsonpath表达式
    :return:
    """
    # 先判断send_request对象中，是否有response属性
    json_path = ','.join(json_path)  # 把元组转换为字符串
    if not hasattr(send_request, "response"):
        log.error(f"$__JSON_EXCUTOR({index},{json_path})$关键字，只能用于post_process或者assert中")
        return
    response = None
    # 获取并判断相应正文中数据是否是json格式
    try:
        response = send_request.response.response_body_dict
    except:
        log.error(f"$__JSON_EXCUTOR({index},{json_path})$关键字，只能用于响应数据为json格式的")
        return
    json_path = json_path.replace('"', "'")
    res_list = jsonpath.jsonpath(response, json_path)
    if res_list:
        index = int(index)
        if index > 0:
            res = res_list[index - 1]
        else:
            res = random.choice(res_list)
        send_request.local_vars[var_name] = res
        log.debug(f"json提取成功，变量: {var_name} 的值为: {res}")


def regex_excutor(send_request, var_name, index, *regex):
    """
    根据正则表达式提取响应中的数据，并存入变量var_name中。index值小于1 表示随机。等于1表示获取第一个结果
    :param send_request: SendRequest对象
    :param var_name: 变量名
    :param index: <=0 表示随机，如果为1 表示获取第1个数据，为2获取第二个数据依次类推
    :param regex: 正则表达式
    :return:
    """
    # 先判断send_request对象中，是否有response属性
    regex = ','.join(regex)  # 把元组转换为字符串
    if not hasattr(send_request, "response"):
        log.error(f"$__REGEX_EXCUTOR({index},{regex})$关键字，只能用于post_process或者assert中")
        return
    text = send_request.response.response_body_text
    try:
        r = re.compile(regex)
        res = r.findall(text)
        if not len(res) == 0:
            a = None
            index = int(index)
            if index < 1:
                a = random.choice(res)
            else:
                a = res[index - 1]
            log.debug(f"正则提取成功，变量: {var_name} 的值为: {a}")
            send_request.local_vars[var_name] = a
    except:
        log.error(f"正则表达式错误：{regex}")


# 断言

def assert_contains(send_request, *expect):
    """
    断言相应正文中是否包含预期结果
    :param send_request:
    :param expect:
    :return:
    """
    expect = ",".join(expect)
    if not hasattr(send_request, "response"):
        log.error(f"$__ASSERT_EXPECT({expect})$关键字，只能用于post_process或者assert中")
        raise
    try:
        assert expect in send_request.response.response_body_text
    except:
        log.error(f"断言失败：响应正文中不包含：{expect}")
        raise
    else:
        log.info(f"断言通过：响应正文中包含：{expect}")


def assert_json_equals(send_request, expect, *json_path):
    """
    断言json_path表达式对应的数据，和预期结果相等
    :param send_request:
    :param expect:
    :param json_path:
    :return:
    """
    # 先判断send_request对象中，是否有response属性
    json_path = ','.join(json_path)  # 把元组转换为字符串
    if not hasattr(send_request, "response"):
        log.error(f"$__ASSERT_JSON({expect},{json_path})$关键字，只能用于post_process或者assert中")
        raise
    response = None

    # 获取并判断相应正文中数据是否是json格式
    try:
        response = send_request.response.response_body_dict
    except:
        log.error(f"$__ASSERT_JSON({expect},{json_path})$关键字，只能用于响应数据为json格式的")
        raise
    json_path = json_path.replace('"', "'")
    res = jsonpath.jsonpath(response, json_path)
    try:
        if res:
            assert str(res[0]) == expect, f"断言失败：jsonpath：{json_path} 对应的数据 {res[0]} 不等于预期结果： {expect}"
        else:
            with allure.step(f"关键字：$__ASSERT_JSON_EQUAL({expect},{json_path})$ 断言结果：不通过"):
                allure.attach(f"断言失败：jsonpath：{json_path} 对应的数据不存在", "断言详情",
                              attachment_type=allure.attachment_type.TEXT)
            assert False, f"断言失败：jsonpath：{json_path} 对应的数据不存在"
    except Exception as e:
        with allure.step(f"关键字：$__ASSERT_JSON_EQUAL({expect},{json_path})$ 断言结果：不通过"):
            allure.attach(f"实际结果：{str(res[0])}  预期结果： {expect}", "断言详情",
                          attachment_type=allure.attachment_type.TEXT)
        log.error(e)
        raise
    else:
        with allure.step(f"关键字：$__ASSERT_JSON_EQUAL({expect},{json_path})$ 断言结果：通过"):
            allure.attach(f"实际结果：{str(res[0])}  预期结果： {expect}", "断言详情",
                          attachment_type=allure.attachment_type.TEXT)
        log.info(f"断言通过：jsonpath：{json_path} 对应的数据 {res[0]} 等于预期结果： {expect}")


def assert_batch(send_request, *expect):
    """
    批量断言
    :param send_request:
    :param *expect: 预期结果，json字符串
    :return:
    """
    expect = ','.join(expect)  # 把元组转换为字符串
    if not hasattr(send_request, "response"):
        log.error(f"$__ASSERT_JSON({expect})$关键字，只能用于post_process或者assert中")
        raise
    response = None
    # 获取并判断相应正文中数据是否是json格式
    try:
        response = send_request.response.response_body_dict
    except:
        log.error(f"$__ASSERT_JSON({expect})$关键字，只能用于响应数据为json格式的")
        raise
    try:
        expect = json.loads(expect)
    except:
        log.error(f"预期结果需要是一个json字符串: {expect}")
        raise

    def compare_json(response, expect):
        '''
        对比预期结果和响应数据
        :param response:
        :param expect:
        :return:
        '''
        if isinstance(expect, dict) and isinstance(response, dict):
            for k in expect:
                if k in response:
                    compare_json(response[k], expect[k])
        elif isinstance(expect, list) and isinstance(response, list):
            for i in range(len(expect)):
                if i <= len(response):
                    compare_json(response[i], expect[i])
        elif isinstance(expect, type(response)):
            try:
                assert expect == response
            except:
                log.error(f"断言失败：实际结果：{response} 和预期结果：{expect} 不一致")
                raise
            else:
                log.info(f"断言通过：实际结果：{response} 和预期结果：{expect} 相等")
        else:
            log.error(f"断言失败：预期结果：{expect} 和实际结果：{response} 数据类型不一致")
            raise AssertionError("断言类型错误")

    compare_json(expect, response)



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : base_request_des.py
# Time       ：2022-04-17 10:36
# Author     ：xuepl
# version    ：python 3.7.1
"""

# !/usr/bin/env python
# -*-coding:utf-8 -*-
from common.descrip import UrlDesc, ParamsDesc, JsonDesc, HeadersDesc, FilesDesc, ConfigDesc, MethodDesc

"""
# File       : base_request.py
# Time       ：2022-04-10 10:52
# Author     ：xuepl
# version    ：python 3.7.1
"""


# 基础请求类

# 目的是给requests框架提供数据
# requests框架，在发送请求时，需要那些参数？
# url:str method:str params:dict data:dict或str json: dict headers:dict file:dict

class BaseRequest():
    url = UrlDesc()
    app = None
    method = MethodDesc()
    params = ParamsDesc()
    data = JsonDesc()
    json = JsonDesc()
    headers = HeadersDesc()
    files = FilesDesc()
    config = ConfigDesc()



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : utils.py
# Time       ：2022-04-10 11:26
# Author     ：xuepl
# version    ：python 3.7.1
"""
import json
import os

import xlrd

from tools import log
from tools.config import Config


def kvstr_to_dict(s):
    d = {}
    try:
        d = json.loads(s)
    except:
        pass
    else:
        return d
    if "=" not in s:
        log.error(f"参数必须是键值对类型: {s}")
        return d
    kvs_list = s.split("&")

    for kvs in kvs_list:
        k, v = kvs.split('=')
        d[k] = v
    return d


def hstr_to_dict(s):
    d = {}
    s = s.strip()  # 去调前后空格
    kvs_list = s.split('\n')
    try:
        d = json.loads(s)
    except:
        pass
    else:
        return d

    for kvs in kvs_list:
        kvs = kvs.strip()
        k, v = kvs.split(": ")
        d[k] = v
    return d


def hdict_to_str(d):
    h_list = []
    for k, v in d.items():
        h_list.append(f"{k}: {v}")
    return "\n".join(h_list)


def read_json_file(file_path):
    res = None
    with open(file_path, 'r', encoding='utf-8') as f:
        res = json.loads(f.read())
    return res


def read_excel(file_path):
    root_path = Config().get_root_path()
    # 打开excel文件
    excel = xlrd.open_workbook(os.path.join(root_path, file_path))
    # 获取excel中所有的sheet
    sheets = excel.sheets()
    case_lsit = []  # 用例数据列表
    ids = []
    for sheet in sheets:
        # 获取当前sheet中，总共有多少行数据
        rows = sheet.nrows
        if rows < 2:
            continue
        # 读取第一行数据
        row_1 = sheet.row_values(1)
        # 使用for循环，读取当前sheet中所有的行
        for i in range(2, rows):
            row = sheet.row_values(i)
            # 把每行内容转换成一个字典，字典的key是对应列第一行数据，值为对应列当前行数据
            d = {}
            for n in range(len(row)):
                d[row_1[n]] = None if row[n] == '' else row[n]
            if d["is_run"] and d["is_run"].strip() == '否':
                continue
            ids.append(d["title"])
            case_lsit.append(d)  # 用例数据放入case_list中
    return ids, case_lsit


def scan_excel(file_path, excel_list):
    files_list = os.listdir(file_path)
    for f in files_list:
        file = os.path.join(file_path, f)  # 路径拼接
        # 判断是否是一个文件夹
        if os.path.isdir(file) and f not in [".git", ".idea", '.pytest_cache', '__pycache__']:
            scan_excel(file, excel_list)
        elif os.path.isfile(file) and (file.endswith(".xls") or file.endswith(".xlsx")):  # 判断是excel文件
            excel_list.append(file)
            pass
        else:
            # 文件路径不存在
            pass


def get_cases():
    root_path = Config().get_root_path()
    excel_list = []
    # 递归结束之后，所有的excel文件路径已经存入excel_list中
    scan_excel(root_path, excel_list)
    # 批量读取excel文件中的数据，并存放至列表中
    ids = []
    cases = []
    for e in excel_list:
        titles, case_lsit = read_excel(e)
        ids.extend(titles)
        cases.extend(case_lsit)
    return ids, cases


if __name__ == '__main__':
    get_cases()



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : mysql_tool.py
# Time       ：2022-04-23 14:16
# Author     ：xuepl
# version    ：python 3.7.1
"""

import pymysql


class MySQLTool():
    def __init__(self, host, user, password, db, port=3306, charset="utf8"):
        self.host = host
        self.user = user
        self.password = password
        self.db = db
        self.port = port
        self.charset = charset

    def connect(self):
        # 建立数据库链接
        self.conn = pymysql.connect(
            host=self.host,
            user=self.user,
            password=self.password,
            db=self.db,
            port=self.port,
            charset=self.charset,
            cursorclass=pymysql.cursors.DictCursor
        )
        # 创建游标
        self.cursor = self.conn.cursor()

    def query(self, sql):
        '''
        查询
        :param sql: sql语句
        :return:
        '''
        self.connect()
        self.cursor.execute(sql)  # 执行sql语句
        res = self.cursor.fetchall()  # 获取所有的查询结果
        self.close()
        return res

    def update(self, sql):
        '''
        增删改
        :param sql:sql语句
        :return:
        '''
        self.connect()
        effect_row = self.cursor.execute(sql)  # 执行sql语句
        self.conn.commit()  # 增删改必须要提交，不然不生效
        self.close()
        return effect_row

    def close(self):
        self.cursor.close()  # 关闭游标
        self.conn.close()  # 关闭数据库链接


if __name__ == '__main__':
    db = MySQLTool(host="api.xuepl.com.cn", user="root", password="SongLin2021", db="mall")
    res = db.query("SELECT * FROM `tb_newbee_mall_user`;")
    print(res)
    # 2022-04-23 14:23:20



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : key_driver.py
# Time       ：2022-04-17 14:09
# Author     ：xuepl
# version    ：python 3.7.1
"""
import json
import re

import allure

from common.key_map import key_maps
from tools import log


class KeyDriver():

    def __init__(self, send_request, value):
        self.send_request = send_request
        self.value = self.to_str(value)
        self.key_replace()

    # 使用json.dumps 把字典或者list类型的数据，转换成字符串

    def to_str(self, value):  # 把所有类型的数据，转换为字符串类型
        if isinstance(value, dict) or isinstance(value, list):
            value = json.dumps(value, ensure_ascii=False)
        else:
            value = str(value)
        return value

    def keys_parser(self, value):
        '''
        返回字符串中所有的关键字
        $__PHONE()$
        $__RANDOM_STR(abceefg,10)$
        $__RANDOM_INT(6,18)$
        $__RANDOM_STR(abceefg,$__RANDOM_INT(6,18)$)$
        参数中包含特殊符号 $ ,
        一个字符串中，可能会包含多个关键字
        :return:
        '''
        length = len(value)  # 获取数据的长度
        start = -1  # 关键字的起始位置，-1 表示没有遇到关键字起始位置
        split_flag = 0  # 解决关键字多层嵌套，定义一个标记。为0表示关键字开始和结束是成对的
        __keys = []  # 关键字列表
        for i in range(length):  # for循环遍历整个字符串
            if (value[i] == "$"):
                # 关键字的起始字符
                if i + 2 < length and value[i:i + 3] == "$__":
                    split_flag -= 1  # 遇到关键字开始字符，split_flag值减一
                    if start < 0:  # 如果已经对关键字起始标志赋值完成，则不需要重复赋值
                        start = i
                # 关键字的结束字符
                elif i - 1 >= 0 and value[i - 1:i + 1] == ")$":
                    split_flag += 1  # 遇到关键字结束字符，split_flag值加一
                    if split_flag == 0:  # 遇到关键字结束字符，如果split_flag为0，则表示关键字完整结束，可以进行切片
                        __keys.append(value[start: i + 1])  # 关键字切片并存入列表中
                        start = -1  # 重置关键字开始标记
        return __keys

    def key_run(self, key):
        # 判断是否存在关键字嵌套
        if "$__" in key[3:-1]:
            # 获取关键字中的内层关键字
            inner_kyes = self.keys_parser(key[3:-1])
            # 循环执行所有的内层关键字
            for i_key in inner_kyes:
                res = self.key_run(i_key)  # 递归执行所有的内层关键字
                key = key.replace(i_key, res)  # 替换关键字中的内层关键字
        key_name = key[3:key.find('(')]  # 获取关键字名称
        r = re.compile(r"\((.*)\)")  # 此处不可以使用非贪婪模式
        key_args = r.findall(key)[0]  # 获取关键字参数
        res = None
        try:
            if key_args == "":  # 如果关键字参数为空
                res = key_maps[key_name](self.send_request)
            else:  # 如果关键字参数不为空
                res = key_maps[key_name](self.send_request, *key_args.split(','))  # *python中的解包赋值
        except AssertionError as e:
            raise
        except:
            log.error(f"关键字：{key}执行失败")
        return self.to_str(res) if res else None  # 把关键字执行结果转换为字符串

    def key_replace(self):
        # 解析数据中所有的关键字
        keys = self.keys_parser(self.value)
        # 把执行的关键字列表，添加进allure报告中
        allure.attach(json.dumps(keys, ensure_ascii=False, indent=2), "关键字列表",
                      attachment_type=allure.attachment_type.TEXT)
        for key in keys:
            # 循环执行关键字
            res = self.key_run(key)
            if res:
                self.value = self.value.replace(key, res)
        return self.value

    def __str__(self):
        # 重写__str__方法，返回value
        return self.value


if __name__ == '__main__':
    s = "adf$__RANDOM_STRING(1234567890,10)$f $__RANDOM_STRING(abceefg,$__RANDOM_INT(6,18)$)$"

    print(KeyDriver(s))



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : base_response.py
# Time       ：2022-04-10 16:35
# Author     ：xuepl
# version    ：python 3.7.1
"""
from tools import log
from tools.utils import hdict_to_str


class BaseResponse():
    def __init__(self, response):
        self.response = response
        self.__print_log()

    @property
    def status_code(self):
        return self.response.status_code

    @property
    def response_headers(self):
        return hdict_to_str(self.response.headers)

    @property
    def response_body_text(self):
        return self.response.text

    @property
    def response_body_dict(self):
        return self.response.json()

    @property
    def response_body_content(self):
        return self.response.content

    @property
    def request_method(self):
        return self.response.request.method

    @property
    def request_url(self):
        return self.response.request.url

    @property
    def request_headers(self):
        return hdict_to_str(self.response.request.headers)

    @property
    def request_body(self):
        return self.response.request.body if self.response.request.body else ""

    def __print_log(self):
        # 打印请求信息
        log.info(f"""
------------------请求报文-----------------
{self.request_method} {self.request_url}
{self.request_headers}

{self.request_body}""")
        # 打印响应信息
        log.info(f"""
---------响应报文-----------------
{self.status_code}
{self.response_headers}

{self.response_body_text}""")

    def __str__(self):
        return f"""
---------响应报文-----------------
{self.status_code}
{self.response_headers}

{self.response_body_text}"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# __title__ = ''
# __author__ = 'xuepl'
# __mtime__ = '2019/9/14'

from distutils.core import setup

setup(
    name="sl-api-tools",  # 这里是pip项目发布的名称 pip install 包名
    version="1.0.3",  # 版本号，数值大的会优先被pip
    keywords=["api", "auto"],
    description="to simplify auto test",
    long_description="A init package,to simplify develope auto test",
    license="MIT Licence",

    url="https://gitee.com/songlinxyb2112/auto-api-xuepl.git",  # 项目相关文件地址，一般是github
    author="xuepl",
    author_email="xuepl@songlinxy.com",
    packages=['tools', "common"],  # 发布的包列表 ,from import的包名
    platforms="python",
    install_requires=[
        'allure-pytest==2.9.45',
        'configparser==5.2.0',
        'Faker==13.3.4',
        'jsonpath==0.82',
        'PyMySQL==1.0.2',
        'pytest==7.0.1',
        'requests==2.27.1',
        'xlrd==2.0.1',
        'xlwt==1.3.0'
    ]
)



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : descrip.py
# Time       ：2022-04-17 11:01
# Author     ：xuepl
# version    ：python 3.7.1
"""

import json
import os

from tools.config import Config
from tools.utils import kvstr_to_dict, hstr_to_dict


# 也可以定义成描述器的写法

# 描述器基类
class BaseDesc():
    __config = None

    @property
    def config(self):
        if self.__config is None:
            __class__.__config = Config()  # 避免后续用例中，多次实例化BaseRequest 类是，对Config类多次实例化
        return self.__config


class MethodDesc(BaseDesc):
    __value = None

    def __get__(self, instance, owner):
        return self.__value

    def __set__(self, instance, value):
        self.__value = value


class ConfigDesc(BaseDesc):

    def __set__(self, instance, value):
        pass

    def __get__(self, instance, owner):
        return self.config


class UrlDesc(BaseDesc):
    __value = None
    __app = None

    def __set__(self, instance, value):
        self.__value = value[0]
        self.__app = value[1]

    def __get__(self, instance, owner):
        return self.config.get_url(self.__app) + self.__value


class ParamsDesc(BaseDesc):
    __value = None

    def __set__(self, instance, value):
        # 对prams数据存储时进行一个格式转换
        if isinstance(value, dict):
            self.__value = value
        elif (isinstance(value, str)):
            self.__value = kvstr_to_dict(value)

    def __get__(self, instance, owner):
        return self.__value


class JsonDesc(BaseDesc):
    __value = None

    def __set__(self, instance, value):
        # 如果json字符串转为字典
        try:
            self.__value = json.loads(value)
        except:
            self.__value = value

    def __get__(self, instance, owner):
        return self.__value


class HeadersDesc(BaseDesc):
    __value = None

    def __set__(self, instance, value):
        # 如果json字符串转为字典,键值对字符串转为字典格式
        if isinstance(value, dict):
            self.__value = value
        elif isinstance(value, str):
            self.__value = hstr_to_dict(value)

    def __get__(self, instance, owner):
        return self.__value


class FilesDesc(BaseDesc):
    __value = None

    def __set__(self, instance, value):
        if not self.__value:
            return
        try:
            if isinstance(self.__value, str):
                self.__value = self.__value.replace("(", "[")
                self.__value = self.__value.replace(")", "]")
            self.__value = json.loads(self.__value)
        except:
            pass
        if isinstance(self.__value, dict):
            for k in self.__value:
                if isinstance(self.__value[k], str):
                    self.__value[k] = open(os.path.join(self.config.get_root_path(), self.__value[k]), 'rb')

        elif isinstance(self.__value, list) or isinstance(self.__value, tuple):
            '''
        [('images', ('foo.png', open('foo.png', 'rb'), 'image/png')),
        ('images', ('bar.png', open('bar.png', 'rb'), 'image/png'))]

        [('images',open('foo.png', 'rb')),
        ('images', open('bar.png', 'rb'))]
        '''
            self.__value = list(self.__value)  # 把元组类型数据，转换为列表类型
            for i in range(len(self.__value)):
                file = list(self.__value[i])
                if isinstance(file[1], str):
                    file[1] = open(os.path.join(self.config.get_root_path(), file[1]), 'rb')  # 二进制格式打开文件
                    self.__value[i] = tuple(file)
                elif isinstance(file[1], tuple) or isinstance(file[1], list):
                    file[1] = list(file[1])
                    file[1][1] = open(os.path.join(self.config.get_root_path(), file[1][1]), 'rb')
                    file[1][1] = tuple(file[1][1])
                    file[1] = tuple(file[1])

        return self.__value

    def __get__(self, instance, owner):
        return self.__value



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : key_map.py
# Time       ：2022-04-17 15:12
# Author     ：xuepl
# version    ：python 3.7.1
"""
from tools.data_tools import random_str, random_int, random_phone
from tools.post_process_tool import json_excutor, regex_excutor, assert_contains, assert_json_equals, assert_batch
from tools.vars_tool import get_var, db, set_var, get_config

key_maps = {
    "RANDOM_STRING": random_str,  # $__RANDOM_STRING(字符取值范围,长度)$ 示例： $__RANDOM_STRING(abcdefg,6)$
    "RANDOM_INT": random_int,  # $__RANDOM_INT(起始,结束)$ 示例： $__RANDOM_INT(6,18)$
    "PHONE": random_phone,  # $__PHONE()$ 示例： $__PHONE()$
    "JSON_EXCUTOR": json_excutor,
    # $__JSON_EXCUTOR(变量名,最终结果中的第几个小于1表示随机大于等于1表示第几个,jsonpath表达式)$ 示例： $__JSON_EXCUTOR(token,0,$.data)$
    "GET_VAR": get_var,  # $__GET_VAR(变量名)$ 示例： $__GET_VAR(变量名)$
    "SET_VAR": set_var,  # $__SET_VAR(变量名,变量值)$ 示例： $__SET_VAR(app,sdfadf)$
    "REGEX_EXCUTOR": regex_excutor,
    # $__REGEX_EXCUTOR(变量名,最终结果中的第几个小于1表示随机大于等于1表示第几个,正则表达式)$ 示例： $__REGEX_EXCUTOR(token,0,"data":"(.*?)")$
    "DB": db,
    # $__DB(应用名,变量名,sql语句)$ 示例： $__DB(mall,userInfo,SELECT login_name,password_md5 FROM `tb_newbee_mall_user`;)$
    "GET_CONFIG": get_config,  # $__GET_CONFIG(section,option)$ 示例： $__GET_CONFIG(user_info,login_name)$
    "ASSERT_CONTAINS": assert_contains,  # $__ASSERT_CONTAINS(预期结果)$  示例： $__ASSERT_CONTAINS("resultCode":200)$
    "ASSERT_JSON_EQUALS": assert_json_equals,
    # $__ASSERT_JSON_EQUAL(预期结果,jsonpath表达式)$  示例： $__ASSERT_JSON_EQUAL(200,$.resultCode)$
    "ASSERT_BATCH": assert_batch,
    # 批量断言$__ASSERT_BATCH(预期结果json字符串)$ 示例：$__ASSERT_BATCH({"resultCode":200,"message":"SUCCESS"})$

}



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : config.py
# Time       ：2022-03-27 17:29
# Author     ：xuepl
# version    ：python 3.7.1
"""
import configparser

# 创建一个初始化器
import os

from tools import log

config = configparser.ConfigParser()

# 获取配置文件的绝对路径
config_path = os.path.join(os.getcwd(), "config.ini")
# 读取配置文件
config.read(config_path, encoding='utf-8')


class Config():
    def __init__(self):
        # 获取环境号
        self.env = os.environ.get("AUTO_ENV")
        # self.env = "te"

    def get_url(self, app_name):
        section = self.env + "_url"
        return self.get_option(section, app_name)

    def get_db(self, app_name):
        # 拼接section
        section = f"{self.env}_{app_name}_db"
        # 根据section获取所有的option
        options = config.options(section)
        # res = {}
        # for o in options:
        #     res[o] = self.get_option(section, o)
        res = {o: self.get_option(section, o) for o in options}  # 字典推导式，根据options获取对应的value，并存入字典中
        res["port"] = int(res["port"])  # 把字符串格式的端口号，强转成数字格式
        return res

    def section_is_exist(self, section):
        return section in config.sections()  # 判断section是否存在

    def get_option(self, section, option):
        try:
            return config.get(section, option)
        except:
            assert False, f"配置文件中获取不到对应的记录,section: {section}, option：{option}"

    def get_root_path(self):
        root_path = self.get_option("root_path", "path")
        if root_path == "":
            root_path = os.path.dirname(config_path)
        return root_path



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : vars_tool.py
# Time       ：2022-04-23 11:14
# Author     ：xuepl
# version    ：python 3.7.1
"""
import jsonpath

from tools import log
from tools.Tripartite_tool import MySQLTool


def db(send_request, app, var_name, *sql):
    """
    执行sql语句，如果是查询语句，并把执行结果存入local_vars字典中,如果是增删改语句，则不会存储执行结果
    :param send_request:
    :param app: 应用名
    :param var_name: 变量名
    :param sql: sql语句
    :return:
    """
    # 1、数据库连接信息
    db_config = send_request.request.config.get_db(app)
    db = MySQLTool(**db_config)
    sql = ",".join(sql)
    sql = sql.strip()  # 去掉前后空格
    if sql[:6].upper() == "SELECT":
        res = db.query(sql)
        log.debug(f"数据库查询成功，变量名为：{var_name} 变量值为：{res}")
        send_request.local_vars[var_name] = res
    else:
        res = db.update(sql)
        log.debug(f"数据库修改成功，影响行数为：{res}")


def get_var(send_request, var_name):
    """
    根据变量名，从字典send_request.local_vars中获取对应的数据
    :param send_request: SendRequest对象
    :param var_name: 变量名
    :return:
    """
    vars = send_request.local_vars
    if not var_name.startswith("$"):
        var_name = "$." + var_name
        var_name = var_name.replace('"', "'")
    res = jsonpath.jsonpath(vars, var_name)
    if res:
        return res[0]


def set_var(send_request, var_name, value):
    """
    把数据存入send_request.local_var 字典中
    :param send_request:
    :param var_name:  变量名
    :param value:  变量值
    :return:
    """
    send_request.local_vars[var_name] = value


'''
$.userInfo[0]['login_name']
$.token
local_vars = {
"token":"asdfsafasdf",
"userInfo":[{'login_name': '134', 'password_md5': ''}, {'login_name': '13818316417', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '13818316418', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '18282101811', 'password_md5': 'e10adc3949ba59abbe56e057f20f883e'}, {'login_name': '13873286446', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '13222105193', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '13240919655', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '15524407288', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '18023601866', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '15584015624', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '18635797377', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '15336801286', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '13371762946', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '15927596993', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '15038517306', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '18195013465', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '13438643969', 'password_md5': '96e79218965eb72c92a549dd5a330112'}, {'login_name': '13781670839', 'password_md5': 'e10adc3949ba59abbe56e057f20f883e'}, {'login_name': '18260154769', 'password_md5': 'e10adc3949ba59abbe56e057f20f883e'}, {'login_name': '14735489575', 'password_md5': 'e10adc3949ba59abbe56e057f20f883e'}, {'login_name': '18077177958', 'password_md5': 'e10adc3949ba59abbe56e057f20f883e'}, {'login_name': '15388781833', 'password_md5': 'e10adc3949ba59abbe56e057f20f883e'}, {'login_name': '15983829075', 'password_md5': 'e10adc3949ba59abbe56e057f20f883e'}, {'login_name': '13353770722', 'password_md5': 'e10adc3949ba59abbe56e057f20f883e'}, {'login_name': '13353770744', 'password_md5': 'e10adc3949ba59abbe56e057f20f883e'}, {'login_name': '13353770755', 'password_md5': 'e10adc3949ba59abbe56e057f20f883e'}]

}
'''


# 封装关键字，从config.ini文件中获取数据

def get_config(send_request, section, option):
    """
    获取config.ini中的数据
    :param send_request:
    :param section:
    :param option:
    :return:
    """
    return send_request.request.config.get_option(section, option)



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : data_tools.py
# Time       ：2022-04-17 15:13
# Author     ：xuepl
# version    ：python 3.7.1
"""
from faker import Faker
import random

fake = Faker("zh_CN")  # 默认英文，初始化为中文


def random_str(send_request, content, length):
    return "".join(random.choices(content, k=int(length)))


def random_int(send_request, start, end):
    return random.randint(int(start), int(end))


def random_phone(send_request):
    return fake.phone_number()


def random_idcard(send_request):
    # 随机身份证号
    pass



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# File       : base_request.py
# Time       ：2022-04-10 10:52
# Author     ：xuepl
# version    ：python 3.7.1
"""

# 基础请求类

# 目的是给requests框架提供数据
# requests框架，在发送请求时，需要那些参数？
# url:str method:str params:dict data:dict或str json: dict headers:dict file:dict
import json
import os

from tools.config import Config
from tools.utils import kvstr_to_dict, hstr_to_dict


# 也可以定义成描述器的写法


class BaseRequest():
    __url = None
    app = None
    method = None
    __params = None
    __data = None
    __json = None
    __headers = None
    __files = None
    __config = None

    @property
    def data(self):
        return self.__data

    @data.setter
    def data(self, value):
        try:
            self.__data = json.loads(value)
        except:
            self.__data = value

    @property
    def json(self):
        return self.__json

    @json.setter
    def json(self, value):
        try:
            self.__json = json.loads(value)
        except:
            self.__json = value

    @property
    def config(self):
        if self.__config is None:
            __class__.__config = Config()  # 避免后续用例中，多次实例化BaseRequest 类是，对Config类多次实例化
        return self.__config

    @property
    def url(self):  # 对url数据进行处理，返回完整的参数
        return self.config.get_url(self.app) + self.__url

    @url.setter
    def url(self, value):
        self.__url = value

    @property
    def params(self):  # 对prams数据返回时进行一个格式转换

        return self.__params

    @params.setter
    def params(self, value):
        # 对prams数据存储时进行一个格式转换
        if isinstance(value, dict):
            self.__params = value
        elif (isinstance(value, str)):
            self.__params = kvstr_to_dict(value)

    @property
    def files(self):  # 对prams数据返回时进行一个格式转换
        if not self.__files:
            return
        try:
            if isinstance(self.__files, str):
                self.__files = self.__files.replace("(", "[")
                self.__files = self.__files.replace(")", "]")
            self.__files = json.loads(self.__files)
        except:
            pass
        if isinstance(self.__files, dict):
            for k in self.__files:
                if isinstance(self.__files[k], str):
                    self.__files[k] = open(os.path.join(self.config.get_root_path(), self.__files[k]), 'rb')

        elif isinstance(self.__files, list) or isinstance(self.__files, tuple):
            '''
        [('images', ('foo.png', open('foo.png', 'rb'), 'image/png')),
        ('images', ('bar.png', open('bar.png', 'rb'), 'image/png'))]
        
        [('images',open('foo.png', 'rb')),
        ('images', open('bar.png', 'rb'))]
        '''
            self.__files = list(self.__files)  # 把元组类型数据，转换为列表类型
            for i in range(len(self.__files)):
                file = list(self.__files[i])
                if isinstance(file[1], str):
                    file[1] = open(os.path.join(self.config.get_root_path(), file[1]), 'rb')  # 二进制格式打开文件
                    self.__files[i] = tuple(file)
                elif isinstance(file[1], tuple) or isinstance(file[1], list):
                    file[1] = list(file[1])
                    file[1][1] = open(os.path.join(self.config.get_root_path(), file[1][1]), 'rb')
                    file[1][1] = tuple(file[1][1])
                    file[1] = tuple(file[1])

        return self.__files

    @files.setter
    def files(self, value):
        self.__files = value

    @property
    def headers(self):  # 对prams数据返回时进行一个格式转换
        return self.__headers

    @headers.setter
    def headers(self, value):
        if isinstance(value, dict):
            self.__headers = value
        elif isinstance(value, str):
            self.__headers = hstr_to_dict(value)
